//////////////////////////////////////////////////////////////////////////////// 
/// Pseudoglobe Screensaver
/// 
/// Copyright  2002  Sebastian Pipping <webmaster@hartwork.org>
/// 
/// -->  http://www.hartwork.org
/// 
/// This source code is released under the GNU General Public License (GPL).
/// See GPL.txt for details. Any non-GPL usage is strictly forbidden.
////////////////////////////////////////////////////////////////////////////////


#include "capplog.h"

CAppLog::CAppLog()
{
	bErrors = FALSE;
	bOpened = FALSE;
}

CAppLog::~CAppLog()
{
	// Check if open
	if( !bOpened )
	{
		return;
	}

	Close();
}

BOOL CAppLog::Open()
{
	// Check if open
	if( bOpened || bErrors )
	{
		return FALSE;
	}

	char * szCommandLine = GetCommandLine();
	char * Begin;
	char * End;
	char szPath[ _MAX_PATH ];

	// Get full path and filename of our file
	if( *szCommandLine == '\"' )
	{
		szCommandLine++;

		// Save address
		Begin = szCommandLine;

		while( *szCommandLine != '\"' )
		{
			szCommandLine++;
		}

		// Save address
		End = szCommandLine - 1;
	}
	else
	{
		// Save address
		Begin = szCommandLine;

		while( *szCommandLine != ' ' )
		{
			szCommandLine++;
		}

		// Save address
		End = szCommandLine - 1;
	}
	strncpy( szPath, Begin, End - Begin + 1 );
	szPath[ End - Begin + 1 ] = '\0';

	// Split path into its components
	char szDrive[ _MAX_DRIVE ];
	char szExt[ _MAX_EXT ];
	char szFilebase[ _MAX_FNAME ];
	char szDir[ _MAX_DIR ];
	_splitpath( szPath, szDrive, szDir, szFilebase, szExt );

	// Get current time
	char szBuffer[ 26 ];
	const time_t Now = time( NULL );
	strftime( szBuffer, 26, "%Y-%m-%d %H%M%S", localtime( &Now ) );

	// Set filename for logfile
	wsprintf( szPath, "%s%s%s.txt", szDrive, szDir, szBuffer );
	szPath[ strlen( szDrive ) + strlen( szDir ) + strlen( szBuffer ) + 4 ] = '\0';

	// Create or overwrite file
	if( !( Stream = fopen( szPath, "w" ) ) )
	{
		// Open failed
		bErrors = TRUE;
		return FALSE;
	}
	bOpened = TRUE;
	AppendTime();
	Append( "\n\n" );

	return TRUE;
}

void CAppLog::Close()
{
	// Check if open
	if( !bOpened )
	{
		return;
	}
	
	// Write finishing time to file
	Append( "\n" );
	AppendTime();

	// Close file
	fclose( Stream );
	Stream = NULL;
}

BOOL CAppLog::Append( const char * szText, ... )
{
	// Check for errors
	if( bErrors )
	{
		return FALSE;
	}

	// Open if necessary
	if( !bOpened )
	{
		Open();
	}

	// Check for errors
	if( !bOpened || bErrors )
	{
		return FALSE;
	}

	// Write to file
	va_list Arglist;
	va_start( Arglist, szText );
	int iResult = vfprintf( Stream, szText, Arglist );
	va_end( Arglist );
	// fputs( szText, Stream );

	if( iResult	< 0 )
	{
		// Writing failed
		bErrors = TRUE;

		// Close file
		fclose( Stream );
		// Stream = NULL;          Why does it fail?

		return FALSE;
	}
	return TRUE;
}

BOOL CAppLog::AppendTime()
{
	if( !bOpened || bErrors )
	{
		return FALSE;
	}

	// Get current time
	char szBuffer[ 20 ];
	const time_t Now = time( NULL );
	strftime( szBuffer, 20, "%Y-%m-%d %H:%M:%S", localtime( &Now ) );

	// Write time to file
	return Append( "%s", szBuffer );
}
